#include <pic.h>				// pic specific identifiers
#define _XTAL_FREQ  4000000		// Xtal speed
__CONFIG(0x3D18);				// Config bits

#define LCD_PORT	PORTA			//
#define LCD_TRIS	TRISA
#define LCD_RS		RA4
#define LCD_RW		RA6
#define LCD_E		RA7

#define JOY_PORT	PORTB			//
#define JOY_TRIS	TRISB
#define PotX		RB6
#define PotY		RB3
#define SW1			RB7
#define SW2			RB2

		// Required prototypes

void LCD_init(void), LCD_cmd(unsigned char ch);
void LCD_goto(char line, char column), LCD_clr(void);
void LCD_cur(unsigned char ch), pulse_E(void), LCD_hex(int value);
void LCD_char(unsigned char ch), LCD_charD(unsigned char ch);
void JOY_ReadSW(void), JOY_ReadX(void), JOY_ReadY(void);
void JOY_init(void), interrupt isr(void);

unsigned char Xtext[] = {"Joy-X "};
unsigned char Ytext[] = {"Joy-Y "};
unsigned char presstext[] = {"Close"};
unsigned char nopresstext[] = {"Open "};

unsigned char HEX_Table[] = {0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36,
							0x37, 0x38, 0x39, 0x41, 0x42, 0x43,
							0x44, 0x45, 0x46};
int X, Y;								// Joystick variables
char flags;
int timer;								// interrupt variable

void main(void)							// program entry
	{
	int index = 0;						// text variable

	OPTION = 0x88;						// timer pre-scaler
	CMCON = 0x7;						// Comparitors off
	LCD_TRIS = 0b00000000;				// Led port as outputs
	JOY_TRIS = 0b11111111;				// Joy stick port
	JOY_init();
	__delay_ms(150);
	RA6 = 0;							// RW low
	LCD_init();							// Initalise screen
	T0IE = 1;
	while(1)							// endless Loop
		{
		JOY_ReadX();							// read X axis
		JOY_ReadY();							// read Y axis
		JOY_ReadSW();							// read switches
		LCD_goto(1,0);							// 1st line
		index = 0;
		while(Xtext[index] != 0)				// while text1[index] is'nt zero
			LCD_char(Xtext[index++]);			// write message 1
		LCD_hex(X/256);							// POT 1
		LCD_hex(X%256);							// count
		LCD_char(' ');
		index = 0;
		if(flags & 1)
			{
			while(presstext[index] != 0)		// while text1[index] is'nt zero
				LCD_char(presstext[index++]);	// write message 3
			}
		else
			{
			while(nopresstext[index] != 0)		// while text1[index] is'nt zero
				LCD_char(nopresstext[index++]);	// write message 4
			}
		LCD_goto(2,0);							// 2nd line
		index = 0;
		while(Ytext[index] != 0)				// while text1[index] is'nt zero
			LCD_char(Ytext[index++]);			// write message 2
		LCD_hex(Y/256);							// POT 2
		LCD_hex(Y%256);							// count
		LCD_char(' ');
		index = 0;
		if(flags & 2)
			{
			while(presstext[index] != 0)		// while text1[index] is'nt zero
				LCD_char(presstext[index++]);	// write message 3
			}
		else
			{
			while(nopresstext[index] != 0)		// while text1[index] is'nt zero
				LCD_char(nopresstext[index++]);	// write message 4
			}
		}
	}

void interrupt isr()
	{
	timer++;						// every interrupt
	T0IF = 0;						// increase timer
	}

void LCD_init()
	{
	LCD_cmd(0x20);					// 4 bit
	LCD_cmd(0x28);					// display shift
	LCD_cmd(0x6);					// character mode
	LCD_cmd(0xc);					// display on / off and cursor
	LCD_clr();						// clear display
	}

void JOY_init()
	{
	JOY_TRIS = 0b10110111;			// set pot axis as outputs
	PotX = 0;						// discharge X axis cap
	PotX = 0;						// discharge Y axis cap
	}

void JOY_ReadSW()
	{
	flags=0;						// switch flags cleared
	if(SW1)
		flags += 1;					// button 1 pressed
	if(SW2)
		flags += 2;					// button 2 pressed
	}

void JOY_ReadX()
	{
	X = 0;
	timer = 0;
	TMR0 = 0;						// clear timer
	JOY_TRIS = 0b11110111;			// bit 6 = pot X axis
	GIE = 1;
    while(!PotX);					// wait for cap to charge
	GIE = 0;
	X = (timer * 256) + TMR0;		// timer counts + TMRO count
	JOY_TRIS = 0b10110111;			// bit 6 = pot Y axis
	PotX = 0;						// discharge cap
	}

void JOY_ReadY()
	{
	Y = 0;
	timer = 0;
	TMR0 = 0;
	JOY_TRIS =0b10111111;			// bit 3 = pot Y axis
	GIE = 1;
    while(!PotY);					// wait for cap to charge
	GIE = 0;
	Y = (timer * 256) + TMR0;		// timer counts + TMRO count
	JOY_TRIS = 0b10110111;			// bit 3 = pot Y axis
	PotY = 0;						// discharge cap
	}

void LCD_hex(int value)
	{
	char data;
	data = value >> 4 & 0xf;
	data = HEX_Table[data]; 				// send upper nibble
	LCD_char(data);
	data = value & 0xf;						// send lower nibble
	data = HEX_Table[data];
	LCD_char(data);
	}

void LCD_goto(char line, char column)		// combines line and lineW
	{
	unsigned char data = 0x80;				// default to 1
	if(line == 2)data = 0xc0;				// change to 2
	data += column;							// add in  column
	LCD_cmd(data);
	}

void LCD_clr()
	{
	LCD_cmd(1);								// Clr screen
	}

void LCD_cur(char on)
	{
	unsigned char cur = 0xc;				// cursor off
	if(on) cur = 0xd;						// cursor on
	LCD_cmd(cur);
	}

void LCD_cmd(unsigned char ch)
	{
	LCD_PORT = ch >> 4 & 0xf;
	LCD_RS = 0;
	pulse_E();
	LCD_PORT = ch & 0xf;
	LCD_RS = 0;
	pulse_E();
	__delay_ms(5);
	}

void LCD_charD(unsigned char ch)
	{
	ch+=0x30;
	LCD_char(ch);
	}

void LCD_char(unsigned char ch)
	{
	LCD_PORT = ch >> 4 & 0xf;
	LCD_RS = 1;
	pulse_E();
	LCD_PORT = ch & 0xf;
	LCD_RS = 1;
	pulse_E();
	__delay_ms(5);;
	}

void pulse_E()
	{
	LCD_E = 1;
	__delay_us(1);
	LCD_E = 0;
	}